@@ -1,4 +1,4 @@
-# Copyright 2005-2016 ECMWF.
+# Copyright 2005-2017 ECMWF.
 #
 # This software is licensed under the terms of the Apache Licence Version 2.0
 # which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -15,19 +15,25 @@ set -ea
 # Check the key(s) in the given grib file have the expected value(s)
 # Assumes the file has only ONE message otherwise output goes on to
 # several lines
-check_key_equals()
+grib_check_key_equals()
 {
-   file=$1
-   key=$2
-   val=$3
-   result=`${tools_dir}grib_get -p $key $file`
-   [ "$result" = "$val" ]
+   a_file=$1
+   a_key=$2
+   a_expected=$3
+   a_result=`${tools_dir}grib_get -p $a_key $a_file`
+   if [ "$a_result" != "$a_expected" ]; then
+      echo "File:     $a_file"
+      echo "Key(s):   $a_key"
+      echo "Expected: $a_expected"
+      echo "Result:   $a_result"
+      exit 1
+   fi
 }
 
-check_key_exists()
+grib_check_key_exists()
 {
-   file=$1
-   key=$2
+   a_file=$1
+   a_key=$2
    # grib_get will fail if the key is not found
-   $tools_dir/grib_get -p $key $file >/dev/null
+   $tools_dir/grib_get -p $a_key $a_file >/dev/null
 }
